@@ -13,8 +13,6 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
-import os
-import re
 from elements import *
 import diagparser
 from utils import XY
@@ -22,13 +20,14 @@ from utils import XY
 
 class DiagramTreeBuilder:
     def build(self, tree):
-        diagram = self.instantiate(Diagram(), tree)
-        for subgroup in diagram.traverse_groups():
+        self.diagram = Diagram()
+        self.instantiate(self.diagram, tree)
+        for subgroup in self.diagram.traverse_groups():
             if len(subgroup.nodes) == 0:
                 subgroup.group.nodes.remove(subgroup)
 
-        self.bind_edges(diagram)
-        return diagram
+        self.bind_edges(self.diagram)
+        return self.diagram
 
     def is_related_group(self, group1, group2):
         if group1.is_parent(group2) or group2.is_parent(group1):
@@ -119,6 +118,12 @@ class DiagramTreeBuilder:
                 name = unquote(stmt.name)
                 Diagram.classes[name] = stmt
 
+            elif isinstance(stmt, diagparser.AttrPlugin):
+                self.diagram.set_plugin(stmt.name, stmt.attrs)
+
+            elif isinstance(stmt, diagparser.Statements):
+                self.instantiate(group, stmt)
+
         group.update_order()
         return group
 
@@ -171,7 +176,6 @@ class DiagramLayoutManager:
         self.adjust_node_order()
 
         height = 0
-        toplevel_nodes = [x for x in self.diagram.nodes if x.xy.x == 0]
         for node in self.diagram.nodes:
             if node.xy.x == 0:
                 self.set_node_height(node, height)
@@ -409,7 +413,7 @@ class DiagramLayoutManager:
                 if prev_child and grandchild > 1 and \
                    not self.is_rhombus(prev_child, child):
                     coord = [p.y for p in self.coordinates if p.x > child.xy.x]
-                    if coord:
+                    if coord and max(coord) >= node.xy.y:
                         height = max(coord) + 1
 
                 while True:
