@@ -48,7 +48,9 @@ def unquote(string):
 class Base(object):
     basecolor = (255, 255, 255)
     textcolor = (0, 0, 0)
+    fontfamily = None
     fontsize = None
+    style = None
     int_attrs = ['colwidth', 'colheight', 'fontsize']
 
     @classmethod
@@ -59,10 +61,20 @@ class Base(object):
     def set_default_text_color(cls, color):
         cls.textcolor = images.color_to_rgb(color)
 
+    @classmethod
+    def set_default_fontfamily(cls, fontfamily):
+        cls.fontfamily = fontfamily
+
+    @classmethod
+    def set_default_fontsize(cls, fontsize):
+        cls.fontsize = int(fontsize)
+
     @classmethod
     def clear(cls):
         cls.basecolor = (255, 255, 255)
         cls.textcolor = (0, 0, 0)
+        cls.fontfamily = None
+        cls.fontsize = None
 
     def duplicate(self):
         return copy.copy(self)
@@ -93,6 +105,14 @@ class Base(object):
         for attr in attrs:
             self.set_attribute(attr)
 
+    def set_style(self, value):
+        if re.search('^(?:none|solid|dotted|dashed|\d+(,\d+)*)$', value, re.I):
+            self.style = value.lower()
+        else:
+            class_name = self.__class__.__name__
+            msg = "WARNING: unknown %s style: %s\n" % (class_name, value)
+            raise AttributeError(msg)
+
 
 class Element(Base):
     namespace = {}
@@ -152,21 +172,31 @@ class Element(Base):
 
 class DiagramNode(Element):
     shape = 'box'
+    linecolor = (0, 0, 0)
+    desctable = []
+    attrname = {}
 
     @classmethod
     def set_default_shape(cls, shape):
         cls.shape = shape
 
+    @classmethod
+    def set_default_linecolor(cls, color):
+        cls.linecolor = images.color_to_rgb(color)
+
     @classmethod
     def clear(cls):
         super(DiagramNode, cls).clear()
         cls.shape = 'box'
+        cls.linecolor = (0, 0, 0)
+        cls.desctable = ['numbered', 'label', 'description']
+        cls.attrname = dict(numbered='No', label='Name',
+                            description='Description')
 
     def __init__(self, id):
         super(DiagramNode, self).__init__(id)
 
         self.label = unquote(id) or ''
-        self.style = None
         self.numbered = None
         self.icon = None
         self.background = None
@@ -179,12 +209,8 @@ class DiagramNode(Element):
         super(DiagramNode, self).set_attribute(attr)
         plugins.fire_node_event(self, 'attr_changed', attr)
 
-    def set_style(self, value):
-        if re.search('^(?:solid|dotted|dashed|\d+(,\d+)*)$', value, re.I):
-            self.style = value.lower()
-        else:
-            msg = "WARNING: unknown node style: %s\n" % value
-            raise AttributeError(msg)
+    def set_linecolor(self, color):
+        self.linecolor = images.color_to_rgb(color)
 
     def set_shape(self, value):
         try:
@@ -211,6 +237,17 @@ class DiagramNode(Element):
     def set_stacked(self, value):
         self.stacked = True
 
+    def to_desctable(self):
+        attrs = []
+        for name in self.desctable:
+            value = getattr(self, name)
+            if value is None:
+                attrs.append(u"")
+            else:
+                attrs.append(getattr(self, name))
+
+        return attrs
+
 
 class NodeGroup(Element):
     basecolor = (243, 152, 0)
@@ -226,6 +263,7 @@ class NodeGroup(Element):
         self.level = 0
         self.separated = False
         self.shape = 'box'
+        self.thick = 3
         self.nodes = []
         self.edges = []
         self.icon = None
@@ -301,6 +339,14 @@ class NodeGroup(Element):
             msg = "WARNING: unknown diagram orientation: %s\n" % value
             raise AttributeError(msg)
 
+    def set_shape(self, value):
+        value = value.lower()
+        if value in ('box', 'line'):
+            self.shape = value
+        else:
+            msg = "WARNING: unknown group shape: %s\n" % value
+            raise AttributeError(msg)
+
 
 class DiagramEdge(Base):
     basecolor = (0, 0, 0)
@@ -390,7 +436,6 @@ class DiagramEdge(Base):
         self.label = None
         self.dir = 'forward'
         self.color = self.basecolor
-        self.style = None
         self.hstyle = None
         self.folded = None
         self.thick = None
@@ -426,13 +471,6 @@ class DiagramEdge(Base):
     def set_color(self, color):
         self.color = images.color_to_rgb(color)
 
-    def set_style(self, value):
-        if re.search('^(?:none|solid|dotted|dashed|\d+(,\d+)*)$', value, re.I):
-            self.style = value.lower()
-        else:
-            msg = "WARNING: unknown edge style: %s\n" % value
-            raise AttributeError(msg)
-
     def set_hstyle(self, value):
         value = value.lower()
         if value in ('generalization', 'composition', 'aggregation'):
@@ -487,7 +525,7 @@ class Diagram(NodeGroup):
 
     classes = {}
     linecolor = (0, 0, 0)
-    int_attrs = ['colwidth', 'colheight', 'fontsize',
+    int_attrs = ['colwidth', 'colheight',
                  'node_width', 'node_height', 'span_width', 'span_height']
 
     @classmethod
@@ -504,9 +542,17 @@ class Diagram(NodeGroup):
         self.span_width = None
         self.span_height = None
         self.page_padding = None
-        self.fontsize = None
         self.edge_layout = None
 
+    def set_plugin(self, name, attrs):
+        try:
+            kwargs = dict([str(unquote(attr.name)), unquote(attr.value)] \
+                          for attr in attrs)
+            plugins.load([name], diagram=self, **kwargs)
+        except:
+            msg = "WARNING: fail to load plugin: %s\n" % name
+            raise AttributeError(msg)
+
     def set_plugins(self, value):
         modules = [name.strip() for name in value.split(',')]
         plugins.load(modules, diagram=self)
@@ -520,6 +566,12 @@ class Diagram(NodeGroup):
             raise AttributeError(msg)
 
     def set_default_text_color(self, color):
+        msg = "WARNING: default_text_color is obsoleted; " + \
+              "use default_textcolor\n"
+        sys.stderr.write(msg)
+        self.set_default_textcolor(color)
+
+    def set_default_textcolor(self, color):
         self.textcolor = images.color_to_rgb(color)
         self._DiagramNode.set_default_text_color(self.textcolor)
         self._NodeGroup.set_default_text_color(self.textcolor)
@@ -530,7 +582,14 @@ class Diagram(NodeGroup):
         self._DiagramNode.set_default_color(color)
 
     def set_default_line_color(self, color):
+        msg = "WARNING: default_line_color is obsoleted; " + \
+              "use default_linecolor\n"
+        sys.stderr.write(msg)
+        self.set_default_linecolor(color)
+
+    def set_default_linecolor(self, color):
         self.linecolor = images.color_to_rgb(color)
+        self._DiagramNode.set_default_linecolor(self.linecolor)
         self._DiagramEdge.set_default_color(self.linecolor)
 
     def set_default_group_color(self, color):
@@ -540,6 +599,16 @@ class Diagram(NodeGroup):
     def set_shape_namespace(self, value):
         noderenderer.set_default_namespace(value)
 
+    def set_default_fontfamily(self, fontfamily):
+        self._DiagramNode.set_default_fontfamily(fontfamily)
+        self._NodeGroup.set_default_fontfamily(fontfamily)
+        self._DiagramEdge.set_default_fontfamily(fontfamily)
+
+    def set_default_fontsize(self, fontsize):
+        self._DiagramNode.set_default_fontsize(fontsize)
+        self._NodeGroup.set_default_fontsize(fontsize)
+        self._DiagramEdge.set_default_fontsize(fontsize)
+
     def set_edge_layout(self, value):
         value = value.lower()
         if value in ('normal', 'flowchart'):
@@ -550,3 +619,8 @@ class Diagram(NodeGroup):
         else:
             msg = "WARNING: unknown edge layout: %s\n" % value
             raise AttributeError(msg)
+
+    def set_fontsize(self, value):
+        msg = "WARNING: fontsize is obsoleted; use default_fontsize\n"
+        sys.stderr.write(msg)
+        self.set_default_fontsize(int(value))
