@@ -15,8 +15,8 @@
 
 import re
 import math
-from itertools import islice, izip, tee
-from blockdiag.utils import ellipse, urlutil, XY
+from itertools import izip, tee
+from blockdiag.utils import ellipse, urlutil, Box
 from blockdiag.utils.myitertools import istep, stepslice
 from blockdiag.utils.PILTextFolder import PILTextFolder as TextFolder
 try:
@@ -101,6 +101,8 @@ def style2cycle(style, thick):
 
 
 class ImageDrawEx(object):
+    self_generative_methods = []
+
     def __init__(self, filename, size, **kwargs):
         if kwargs.get('im'):
             self.image = kwargs.get('im')
@@ -115,15 +117,12 @@ class ImageDrawEx(object):
         self.scale_ratio = kwargs.get('scale_ratio', 1)
         self.mode = kwargs.get('mode')
         self.draw = ImageDraw.ImageDraw(self.image, self.mode)
-        self.fontpath = None
-        self.fontsize = None
 
-    def set_default_font(self, fontpath, fontsize):
-        self.fontpath = fontpath
-        self.fontsize = fontsize
+        if 'parent' in kwargs:
+            parent = kwargs['parent']
+            self.scale_ratio = parent.scale_ratio
 
     def resizeCanvas(self, size):
-        image = self.image.resize(size, Image.ANTIALIAS)
         self.image = self.image.resize(size, Image.ANTIALIAS)
         self.draw = ImageDraw.ImageDraw(self.image, self.mode)
 
@@ -247,12 +246,11 @@ class ImageDrawEx(object):
             del kwargs['outline']
             self.line(xy, **kwargs)
 
-    def text(self, xy, string, **kwargs):
+    def text(self, xy, string, font, **kwargs):
         fill = kwargs.get('fill')
 
-        if self.fontpath:
-            fontsize = kwargs.get('fontsize') or self.fontsize
-            ttfont = ImageFont.truetype(self.fontpath, fontsize)
+        if font.path:
+            ttfont = ImageFont.truetype(font.path, font.size)
         else:
             ttfont = None
 
@@ -285,22 +283,32 @@ class ImageDrawEx(object):
 
             self.draw = ImageDraw.ImageDraw(self.image, self.mode)
 
-    def textarea(self, box, string, **kwargs):
-        if 'fontsize' in kwargs:
-            fontsize = kwargs.get('fontsize') or self.fontsize
-            del kwargs['fontsize']
-        else:
-            fontsize = self.fontsize
+    def textarea(self, box, string, font, **kwargs):
+        if 'rotate' in kwargs and kwargs['rotate'] != 0:
+            angle = 360 - kwargs['rotate']
+            del kwargs['rotate']
 
-        lines = TextFolder(box, string, scale=self.scale_ratio,
-                           font=self.fontpath, fontsize=fontsize, **kwargs)
+            if angle in (90, 270):
+                _box = Box(0, 0, box.height, box.width)
+            else:
+                _box = box
+
+            text = ImageDrawEx(None, _box.size, parent=self, mode=self.mode)
+            text.textarea((0, 0, _box.width, _box.height), string, **kwargs)
+
+            filter = Image.new('RGB', box.size, kwargs.get('fill'))
+            self.image.paste(filter, box.topleft, text.image.rotate(angle))
+            self.draw = ImageDraw.ImageDraw(self.image, self.mode)
+            return
+
+        lines = TextFolder(box, string, font, scale=self.scale_ratio, **kwargs)
 
         if kwargs.get('outline'):
             outline = kwargs.get('outline')
             self.rectangle(lines.outlinebox, fill='white', outline=outline)
 
         for string, xy in lines.lines:
-            self.text(xy, string, fontsize=fontsize, **kwargs)
+            self.text(xy, string, font, **kwargs)
 
     def loadImage(self, filename, box):
         box_width = box[2] - box[0]
