@@ -14,11 +14,9 @@
 #  limitations under the License.
 
 import re
-import sys
 import base64
-from blockdiag.utils import urlutil, XY
+from blockdiag.utils import urlutil, Box, XY
 from simplesvg import *
-import xml.sax.saxutils
 
 try:
     from blockdiag.utils.PILTextFolder import PILTextFolder as TextFolder
@@ -29,12 +27,10 @@ feGaussianBlur = svgclass('feGaussianBlur')
 
 
 class SVGImageDrawElement(object):
-    def __init__(self, svg):
-        self.svg = svg
+    self_generative_methods = ['group']
 
-    def set_default_font(self, fontpath, fontsize):
-        self.fontpath = fontpath
-        self.fontsize = fontsize
+    def __init__(self, svg, parent=None):
+        self.svg = svg
 
     def rgb(self, color):
         if isinstance(color, tuple):
@@ -52,31 +48,38 @@ class SVGImageDrawElement(object):
 
         return filter
 
-    def style(self, name):
+    def style(self, name, thick):
+        if thick is None:
+            thick = 1
+
         if name == 'dotted':
-            length = 2
+            length = 2 * thick
         elif name == 'dashed':
-            length = 4
+            length = 4 * thick
+        elif name == 'none':
+            length = "%d %d" % (0, 65535 * thick)
         elif re.search('^\d+(,\d+)*$', name or ""):
-            length = re.sub(',', ' ', name)
+            l = [int(n) * thick for n in name.split(",")]
+            length = " ".join(str(n) for n in l)
         else:
             length = None
 
         return length
 
     def path(self, pd, **kwargs):
+        thick = kwargs.get('thick')
         fill = kwargs.get('fill')
         outline = kwargs.get('outline')
         style = kwargs.get('style')
         filter = kwargs.get('filter')
 
         p = path(pd, fill=self.rgb(fill), stroke=self.rgb(outline),
-                 stroke_dasharray=self.style(style),
+                 stroke_dasharray=self.style(style, thick),
                  style=self.filter(filter))
         self.svg.addElement(p)
 
     def rectangle(self, box, **kwargs):
-        thick = kwargs.get('width')
+        thick = kwargs.get('thick')
         fill = kwargs.get('fill', 'none')
         outline = kwargs.get('outline')
         style = kwargs.get('style')
@@ -89,34 +92,53 @@ class SVGImageDrawElement(object):
 
         r = rect(x, y, width, height, fill=self.rgb(fill),
                  stroke=self.rgb(outline), stroke_width=thick,
-                 stroke_dasharray=self.style(style),
+                 stroke_dasharray=self.style(style, thick),
                  style=self.filter(filter))
         self.svg.addElement(r)
 
-    def text(self, xy, string, **kwargs):
+    def text(self, xy, string, font, **kwargs):
         fill = kwargs.get('fill')
-        fontsize = kwargs.get('fontsize') or self.fontsize
 
-        t = text(xy[0], xy[1], string,
-                 font_size=fontsize, fill=self.rgb(fill))
+        t = text(xy[0], xy[1], string, fill=self.rgb(fill),
+                 font_family=font.generic_family, font_size=font.size,
+                 font_weight=font.weight, font_style=font.style)
         self.svg.addElement(t)
 
-    def textarea(self, box, string, **kwargs):
-        if 'fontsize' in kwargs:
-            fontsize = kwargs.get('fontsize') or self.fontsize
-            del kwargs['fontsize']
-        else:
-            fontsize = self.fontsize
+    def textarea(self, box, string, font, **kwargs):
+        if 'rotate' in kwargs and kwargs['rotate'] != 0:
+            angle = int(kwargs['rotate'])
+            del kwargs['rotate']
 
-        lines = TextFolder(box, string, adjustBaseline=True,
-                           font=self.fontpath, fontsize=fontsize, **kwargs)
+            if angle in (90, 270):
+                _box = Box(box[0], box[1],
+                           box[0] + box.height, box[1] + box.width)
+                if angle == 90:
+                    _box = _box.shift(x=box.width)
+                elif angle == 270:
+                    _box = _box.shift(y=box.height)
+            elif angle == 180:
+                _box = Box(box[2], box[3],
+                           box[2] + box.width, box[3] + box.height)
+            else:
+                _box = Box(box[0], box[1],
+                           box[0] + box.width, box[1] + box.height)
+
+            rotate = "rotate(%d,%d,%d)" % (angle, _box[0], _box[1])
+            group = g(transform="%s" % rotate)
+            self.svg.addElement(group)
+
+            elem = SVGImageDrawElement(group, self)
+            elem.textarea(_box, string, font, **kwargs)
+            return
+
+        lines = TextFolder(box, string, font, adjustBaseline=True, **kwargs)
 
         if kwargs.get('outline'):
             outline = kwargs.get('outline')
             self.rectangle(lines.outlinebox, fill='white', outline=outline)
 
         for string, xy in lines.lines:
-            self.text(xy, string, fontsize=fontsize, **kwargs)
+            self.text(xy, string, font, **kwargs)
 
     def line(self, xy, **kwargs):
         fill = kwargs.get('fill')
@@ -128,10 +150,11 @@ class SVGImageDrawElement(object):
             pd.line(pt.x, pt.y)
 
         p = path(pd, fill="none", stroke=self.rgb(fill),
-                 stroke_width=thick, stroke_dasharray=self.style(style))
+                 stroke_width=thick, stroke_dasharray=self.style(style, thick))
         self.svg.addElement(p)
 
     def arc(self, xy, start, end, **kwargs):
+        thick = kwargs.get('thick')
         fill = kwargs.get('fill')
         style = kwargs.get('style')
 
@@ -161,10 +184,11 @@ class SVGImageDrawElement(object):
         pd = pathdata(pt1[0], pt1[1])
         pd.ellarc(w, h, 0, largearc, 1, pt2[0], pt2[1])
         p = path(pd, fill="none", stroke=self.rgb(fill),
-                 stroke_dasharray=self.style(style))
+                 stroke_dasharray=self.style(style, thick))
         self.svg.addElement(p)
 
     def ellipse(self, xy, **kwargs):
+        thick = kwargs.get('thick')
         fill = kwargs.get('fill')
         outline = kwargs.get('outline')
         style = kwargs.get('style')
@@ -176,18 +200,19 @@ class SVGImageDrawElement(object):
 
         e = ellipse(pt.x, pt.y, w, h, fill=self.rgb(fill),
                     stroke=self.rgb(outline),
-                    stroke_dasharray=self.style(style),
+                    stroke_dasharray=self.style(style, thick),
                     style=self.filter(filter))
         self.svg.addElement(e)
 
     def polygon(self, xy, **kwargs):
+        thick = kwargs.get('thick')
         fill = kwargs.get('fill')
         outline = kwargs.get('outline')
         style = kwargs.get('style')
         filter = kwargs.get('filter')
 
         pg = polygon(xy, fill=self.rgb(fill), stroke=self.rgb(outline),
-                     stroke_dasharray=self.style(style),
+                     stroke_dasharray=self.style(style, thick),
                      style=self.filter(filter))
         self.svg.addElement(pg)
 
@@ -206,6 +231,12 @@ class SVGImageDrawElement(object):
         im = image(url, x, y, w, h)
         self.svg.addElement(im)
 
+    def group(self):
+        group = g()
+        self.svg.addElement(group)
+
+        return SVGImageDrawElement(group)
+
 
 class SVGImageDraw(SVGImageDrawElement):
     def __init__(self, filename, size, **kwargs):
